@@ -37,23 +37,21 @@
 
 	@$csrf_token = $_REQUEST['csrf_token'];
 
-	require_once "functions.php";
 	require_once "sessions.php";
+	require_once "functions.php";
 	require_once "config.php";
 	require_once "db.php";
 	require_once "db-prefs.php";
 
-	no_cache_incantation();
-
 	startup_gettext();
 
-	$script_started = getmicrotime();
+	$script_started = microtime(true);
 
 	$link = db_connect(DB_HOST, DB_USER, DB_PASS, DB_NAME);
 
 	if (!init_connection($link)) return;
 
-	header("Content-Type: text/plain; charset=utf-8");
+	header("Content-Type: text/json; charset=utf-8");
 
 	if (ENABLE_GZIP_OUTPUT && function_exists("ob_gzhandler")) {
 		ob_start("ob_gzhandler");
@@ -63,6 +61,10 @@
 		authenticate_user($link, "admin", null);
 	}
 
+	if ($_SESSION["uid"]) {
+		load_user_plugins($link, $_SESSION["uid"]);
+	}
+
 	$purge_intervals = array(
 		0  => __("Use default"),
 		-1 => __("Never purge"),
@@ -93,17 +95,6 @@
 		1440 => __("Daily"),
 		10080 => __("Weekly"));
 
-	$update_methods = array(
-		0   => __("Default"),
-		1   => __("Magpie"),
-		2   => __("SimplePie"));
-
-	if (DEFAULT_UPDATE_METHOD == "1") {
-		$update_methods[0] .= ' (SimplePie)';
-	} else {
-		$update_methods[0] .= ' (Magpie)';
-	}
-
 	$access_level_names = array(
 		0 => __("User"),
 		5 => __("Power User"),
@@ -118,31 +109,43 @@
 
 	$op = str_replace("-", "_", $op);
 
-	if (class_exists($op)) {
-		$handler = new $op($link, $_REQUEST);
+	global $pluginhost;
+	$override = $pluginhost->lookup_handler($op, $method);
 
-		if ($handler && is_subclass_of($handler, 'Handler')) {
+	if (class_exists($op) || $override) {
+
+		if ($override) {
+			$handler = $override;
+		} else {
+			$handler = new $op($link, $_REQUEST);
+		}
+
+		if ($handler && implements_interface($handler, 'IHandler')) {
 			if (validate_csrf($csrf_token) || $handler->csrf_ignore($method)) {
 				if ($handler->before($method)) {
 					if ($method && method_exists($handler, $method)) {
 						$handler->$method();
+					} else {
+						if (method_exists($handler, "catchall")) {
+							$handler->catchall($method);
+						}
 					}
 					$handler->after();
 					return;
 				} else {
-					header("Content-Type: text/plain");
+					header("Content-Type: text/json");
 					print json_encode(array("error" => array("code" => 6)));
 					return;
 				}
 			} else {
-				header("Content-Type: text/plain");
+				header("Content-Type: text/json");
 				print json_encode(array("error" => array("code" => 6)));
 				return;
 			}
 		}
 	}
 
-	header("Content-Type: text/plain");
+	header("Content-Type: text/json");
 	print json_encode(array("error" => array("code" => 7)));
 
 	// We close the connection to database.
