@@ -4,7 +4,7 @@ require_once "colors.php";
 class Feeds extends Handler_Protected {
 
 	function csrf_ignore($method) {
-		$csrf_ignored = array("index");
+		$csrf_ignored = array("index", "feedbrowser", "quickaddfeed", "search");
 
 		return array_search($method, $csrf_ignored) !== false;
 	}
@@ -22,7 +22,7 @@ class Feeds extends Handler_Protected {
 	}
 
 	private function format_headline_subtoolbar($feed_site_url, $feed_title,
-			$feed_id, $is_cat, $search, $match_on,
+			$feed_id, $is_cat, $search,
 			$search_mode, $view_mode, $error) {
 
 		$page_prev_link = "viewFeedGoPage(-1)";
@@ -50,7 +50,7 @@ class Feeds extends Handler_Protected {
 		if ($is_cat) $cat_q = "&is_cat=$is_cat";
 
 		if ($search) {
-			$search_q = "&q=$search&m=$match_on&smode=$search_mode";
+			$search_q = "&q=$search&smode=$search_mode";
 		} else {
 			$search_q = "";
 		}
@@ -81,8 +81,8 @@ class Feeds extends Handler_Protected {
 		$reply .= "
 			<a href=\"#\"
 				title=\"".__("View as RSS feed")."\"
-				onclick=\"displayDlg('generatedFeed', '$feed_id:$is_cat:$rss_link')\">
-				<img class=\"noborder\" style=\"vertical-align : middle\" src=\"images/feed-icon-12x12.png\"></a>";
+				onclick=\"displayDlg('".__("View as RSS")."','generatedFeed', '$feed_id:$is_cat:$rss_link')\">
+				<img class=\"noborder\" style=\"vertical-align : middle\" src=\"images/pub_set.svg\"></a>";
 
 		$reply .= "</span>";
 
@@ -98,7 +98,7 @@ class Feeds extends Handler_Protected {
 
 		$reply .= "<select dojoType=\"dijit.form.Select\"
 			onchange=\"headlineActionsChange(this)\">";
-		$reply .= "<option value=\"false\">".__('Actions...')."</option>";
+		$reply .= "<option value=\"false\">".__('More...')."</option>";
 
 		$reply .= "<option value=\"0\" disabled=\"1\">".__('Selection toggle:')."</option>";
 
@@ -119,14 +119,23 @@ class Feeds extends Handler_Protected {
 
 		}
 
-		$reply .= "<option value=\"emailArticle(false)\">".__('Forward by email').
-			"</option>";
+		global $pluginhost;
+
+		if ($pluginhost->get_plugin("mail")) {
+			$reply .= "<option value=\"emailArticle(false)\">".__('Forward by email').
+				"</option>";
+		}
+
+		if ($pluginhost->get_plugin("mailto")) {
+			$reply .= "<option value=\"mailtoArticle(false)\">".__('Forward by email').
+				"</option>";
+		}
 
 		$reply .= "<option value=\"0\" disabled=\"1\">".__('Feed:')."</option>";
 
-		$reply .= "<option value=\"catchupPage()\">".__('Mark as read')."</option>";
+		//$reply .= "<option value=\"catchupPage()\">".__('Mark as read')."</option>";
 
-		$reply .= "<option value=\"displayDlg('generatedFeed', '$feed_id:$is_cat:$rss_link')\">".__('View as RSS')."</option>";
+		$reply .= "<option value=\"displayDlg('".__("View as RSS")."','generatedFeed', '$feed_id:$is_cat:$rss_link')\">".__('View as RSS')."</option>";
 
 		$reply .= "</select>";
 
@@ -134,6 +143,10 @@ class Feeds extends Handler_Protected {
 
 		//$reply .= "</h2";
 
+		foreach ($pluginhost->get_hooks($pluginhost::HOOK_HEADLINE_TOOLBAR_BUTTON) as $p) {
+			 echo $p->hook_headline_toolbar_button($feed_id, $is_cat);
+		}
+
 		return $reply;
 	}
 
@@ -141,11 +154,14 @@ class Feeds extends Handler_Protected {
 					$next_unread_feed, $offset, $vgr_last_feed = false,
 					$override_order = false, $include_children = false) {
 
+		if (isset($_REQUEST["DevForceUpdate"]))
+			header("Content-Type: text/plain");
+
 		$disable_cache = false;
 
 		$reply = array();
 
-		$timing_info = getmicrotime();
+		$timing_info = microtime(true);
 
 		$topmost_article_ids = array();
 
@@ -158,15 +174,14 @@ class Feeds extends Handler_Protected {
 			// Update the feed if required with some basic flood control
 
 			$result = db_query($this->link,
-				"SELECT cache_images,cache_content,".SUBSTRING_FOR_DATE."(last_updated,1,19) AS last_updated
+				"SELECT cache_images,".SUBSTRING_FOR_DATE."(last_updated,1,19) AS last_updated
 					FROM ttrss_feeds WHERE id = '$feed'");
 
 				if (db_num_rows($result) != 0) {
 					$last_updated = strtotime(db_fetch_result($result, 0, "last_updated"));
 					$cache_images = sql_bool_to_bool(db_fetch_result($result, 0, "cache_images"));
-					$cache_content = sql_bool_to_bool(db_fetch_result($result, 0, "cache_content"));
 
-					if (!$cache_images && !$cache_content && time() - $last_updated > 120 || isset($_REQUEST['DevForceUpdate'])) {
+					if (!$cache_images && time() - $last_updated > 120 || isset($_REQUEST['DevForceUpdate'])) {
 						include "rssfuncs.php";
 						update_rss_feed($this->link, $feed, true, true);
 					} else {
@@ -191,35 +206,13 @@ class Feeds extends Handler_Protected {
 			}
 		}
 
-		if (is_numeric($feed) && $feed > 0) {
-
-			$result = db_query($this->link, "SELECT rtl_content FROM ttrss_feeds
-				WHERE id = '$feed' AND owner_uid = " . $_SESSION["uid"]);
-
-			if (db_num_rows($result) == 1) {
-				$rtl_content = sql_bool_to_bool(db_fetch_result($result, 0, "rtl_content"));
-			} else {
-				$rtl_content = false;
-			}
-
-			if ($rtl_content) {
-				$rtl_tag = "dir=\"RTL\"";
-			} else {
-				$rtl_tag = "";
-			}
-		} else {
-			$rtl_tag = "";
-			$rtl_content = false;
-		}
-
-		@$search = db_escape_string($_REQUEST["query"]);
+		@$search = db_escape_string($this->link, $_REQUEST["query"]);
 
 		if ($search) {
 			$disable_cache = true;
 		}
 
-		@$search_mode = db_escape_string($_REQUEST["search_mode"]);
-		$match_on = "both"; // deprecated, TODO: remove
+		@$search_mode = db_escape_string($this->link, $_REQUEST["search_mode"]);
 
 		if ($_REQUEST["debug"]) $timing_info = print_checkpoint("H0", $timing_info);
 
@@ -228,9 +221,38 @@ class Feeds extends Handler_Protected {
 		    $search_mode = $method;
 		}
 //		error_log("search_mode: " . $search_mode);
-		$qfh_ret = queryFeedHeadlines($this->link, $feed, $limit, $view_mode, $cat_view,
-			$search, $search_mode, $match_on, $override_order, $offset, 0,
-			false, 0, $include_children);
+
+		if (!$cat_view && is_numeric($feed) && $feed < PLUGIN_FEED_BASE_INDEX && $feed > LABEL_BASE_INDEX) {
+			global $pluginhost;
+
+			$handler = $pluginhost->get_feed_handler(
+				PluginHost::feed_to_pfeed_id($feed));
+
+		//	function queryFeedHeadlines($link, $feed, $limit, $view_mode, $cat_view, $search, $search_mode, $override_order = false, $offset = 0, $owner_uid = 0, $filter = false, $since_id = 0, $include_children = false, $ignore_vfeed_group = false) {
+
+			if ($handler) {
+				$options = array(
+					"limit" => $limit,
+					"view_mode" => $view_mode,
+					"cat_view" => $cat_view,
+					"search" => $search,
+					"search_mode" => $search_mode,
+					"override_order" => $override_order,
+					"offset" => $offset,
+					"owner_uid" => $_SESSION["uid"],
+					"filter" => false,
+					"since_id" => 0,
+					"include_children" => $include_children);
+
+				$qfh_ret = $handler->get_headlines(PluginHost::feed_to_pfeed_id($feed),
+					$options);
+			}
+
+		} else {
+			$qfh_ret = queryFeedHeadlines($this->link, $feed, $limit, $view_mode, $cat_view,
+				$search, $search_mode, $override_order, $offset, 0,
+				false, 0, $include_children);
+		}
 
 		if ($_REQUEST["debug"]) $timing_info = print_checkpoint("H1", $timing_info);
 
@@ -238,18 +260,17 @@ class Feeds extends Handler_Protected {
 		$feed_title = $qfh_ret[1];
 		$feed_site_url = $qfh_ret[2];
 		$last_error = $qfh_ret[3];
-		$cache_content = true;
 
 		$vgroup_last_feed = $vgr_last_feed;
 
 		$reply['toolbar'] = $this->format_headline_subtoolbar($feed_site_url,
 			$feed_title,
-			$feed, $cat_view, $search, $match_on, $search_mode, $view_mode,
+			$feed, $cat_view, $search, $search_mode, $view_mode,
 			$last_error);
 
 		$headlines_count = db_num_rows($result);
 
-		if (get_pref($this->link, 'COMBINED_DISPLAY_MODE')) {
+		/* if (get_pref($this->link, 'COMBINED_DISPLAY_MODE')) {
 			$button_plugins = array();
 			foreach (explode(",", ARTICLE_BUTTON_PLUGINS) as $p) {
 				$pclass = "button_" . trim($p);
@@ -259,7 +280,9 @@ class Feeds extends Handler_Protected {
 					array_push($button_plugins, $plugin);
 				}
 			}
-		}
+		} */
+
+		global $pluginhost;
 
 		if (db_num_rows($result) > 0) {
 
@@ -272,6 +295,8 @@ class Feeds extends Handler_Protected {
 
 			if ($_REQUEST["debug"]) $timing_info = print_checkpoint("PS", $timing_info);
 
+			$expand_cdm = get_pref($this->link, 'CDM_EXPANDED');
+
 			while ($line = db_fetch_assoc($result)) {
 				$class = ($lnum % 2) ? "even" : "odd";
 
@@ -318,23 +343,6 @@ class Feeds extends Handler_Protected {
 					array_push($topmost_article_ids, $id);
 				}
 
-				if ($line["last_read"] == "" && !sql_bool_to_bool($line["unread"])) {
-
-					$update_pic = "<img id='FUPDPIC-$id' src=\"".
-						theme_image($this->link, 'images/updated.png')."\"
-						alt=\"Updated\">";
-				} else {
-					$update_pic = "<img id='FUPDPIC-$id' src=\"images/blank_icon.gif\"
-						alt=\"Updated\">";
-				}
-
-				if (sql_bool_to_bool($line["unread"]) &&
-					time() - strtotime($line["updated_noms"]) < $fresh_intl) {
-
-					$update_pic = "<img id='FUPDPIC-$id' src=\"".
-						theme_image($this->link, 'images/fresh_sign.png')."\" alt=\"Fresh\">";
-				}
-
 				if ($line["unread"] == "t" || $line["unread"] == "1") {
 					$class .= " Unread";
 					++$num_unread;
@@ -345,24 +353,22 @@ class Feeds extends Handler_Protected {
 
 				if ($line["marked"] == "t" || $line["marked"] == "1") {
 					$marked_pic = "<img id=\"FMPIC-$id\"
-						src=\"".theme_image($this->link, 'images/mark_set.png')."\"
+						src=\"images/mark_set.svg\"
 						class=\"markedPic\" alt=\"Unstar article\"
 						onclick='javascript:toggleMark($id)'>";
 				} else {
 					$marked_pic = "<img id=\"FMPIC-$id\"
-						src=\"".theme_image($this->link, 'images/mark_unset.png')."\"
+						src=\"images/mark_unset.svg\"
 						class=\"markedPic\" alt=\"Star article\"
 						onclick='javascript:toggleMark($id)'>";
 				}
 
 				if ($line["published"] == "t" || $line["published"] == "1") {
-					$published_pic = "<img id=\"FPPIC-$id\" src=\"".theme_image($this->link,
-						'images/pub_set.png')."\"
+					$published_pic = "<img id=\"FPPIC-$id\" src=\"images/pub_set.svg\"
 						class=\"markedPic\"
 						alt=\"Unpublish article\" onclick='javascript:togglePub($id)'>";
 				} else {
-					$published_pic = "<img id=\"FPPIC-$id\" src=\"".theme_image($this->link,
-						'images/pub_unset.png')."\"
+					$published_pic = "<img id=\"FPPIC-$id\" src=\"images/pub_unset.svg\"
 						class=\"markedPic\"
 						alt=\"Publish article\" onclick='javascript:togglePub($id)'>";
 				}
@@ -378,7 +384,9 @@ class Feeds extends Handler_Protected {
 #				$content_link = "<a href=\"javascript:viewContentUrl('".$line["link"]."');\">" .
 #					$line["title"] . "</a>";
 
-				$updated_fmt = make_local_datetime($this->link, $line["updated_noms"], false);
+				$updated_fmt = make_local_datetime($this->link, $line["updated"], false);
+				$date_entered_fmt = T_sprintf("Imported at %s",
+					make_local_datetime($this->link, $line["date_entered"], false));
 
 				if (get_pref($this->link, 'SHOW_CONTENT_PREVIEW')) {
 					$content_preview = truncate_string(strip_tags($line["content_preview"]),
@@ -387,8 +395,7 @@ class Feeds extends Handler_Protected {
 
 				$score = $line["score"];
 
-				$score_pic = theme_image($this->link,
-					"images/" . get_score_pic($score));
+				$score_pic = "images/" . get_score_pic($score);
 
 /*				$score_title = __("(Click to change)");
 				$score_pic = "<img class='hlScorePic' src=\"images/$score_pic\"
@@ -416,9 +423,11 @@ class Feeds extends Handler_Protected {
 				if ($has_feed_icon) {
 					$feed_icon_img = "<img class=\"tinyFeedIcon\" src=\"".ICONS_URL."/$feed_id.ico\" alt=\"\">";
 				} else {
-					$feed_icon_img = "<img class=\"tinyFeedIcon\" src=\"images/feed-icon-12x12.png\" alt=\"\">";
+					$feed_icon_img = "<img class=\"tinyFeedIcon\" src=\"images/pub_set.svg\" alt=\"\">";
 				}
 
+				$entry_site_url = $line["site_url"];
+
 				if (!get_pref($this->link, 'COMBINED_DISPLAY_MODE')) {
 
 					if (get_pref($this->link, 'VFEED_GROUP_BY_FEED')) {
@@ -429,26 +438,25 @@ class Feeds extends Handler_Protected {
 
 							$cur_feed_title = htmlspecialchars($cur_feed_title);
 
-							$vf_catchup_link = "(<a onclick='catchupFeedInGroup($feed_id);' href='#'>".__('mark as read')."</a>)";
+							$vf_catchup_link = "(<a class='catchup' onclick='catchupFeedInGroup($feed_id);' href='#'>".__('Mark as read')."</a>)";
 
 							$reply['content'] .= "<div class='cdmFeedTitle'>".
 								"<div style=\"float : right\">$feed_icon_img</div>".
-								"<a href=\"#\" onclick=\"viewfeed($feed_id)\">".
+								"<a class='title' href=\"#\" onclick=\"viewfeed($feed_id)\">".
 								$line["feed_title"]."</a> $vf_catchup_link</div>";
 
 						}
 					}
 
-					$mouseover_attrs = "onmouseover='postMouseIn($id)'
+					$mouseover_attrs = "onmouseover='postMouseIn(event, $id)'
 						onmouseout='postMouseOut($id)'";
 
-					$reply['content'] .= "<div class='$class' id='RROW-$id' $label_row_style $mouseover_attrs>";
-
-					$reply['content'] .= "<div class='hlUpdPic'>$update_pic</div>";
+					$reply['content'] .= "<div class='hl $class' id='RROW-$id' $label_row_style $mouseover_attrs>";
 
 					$reply['content'] .= "<div class='hlLeft'>";
 
-					$reply['content'] .= "<input type=\"checkbox\" onclick=\"tSR(this)\"
+					$reply['content'] .= "<input dojoType=\"dijit.form.CheckBox\"
+							type=\"checkbox\" onclick=\"toggleSelectRow2(this)\"
 							id=\"RCHK-$id\">";
 
 					$reply['content'] .= "$marked_pic";
@@ -458,7 +466,7 @@ class Feeds extends Handler_Protected {
 
 					$reply['content'] .= "<div onclick='return hlClicked(event, $id)'
 						class=\"hlTitle\"><span class='hlContent$hlc_suffix'>";
-					$reply['content'] .= "<a id=\"RTITLE-$id\"
+					$reply['content'] .= "<a id=\"RTITLE-$id\" class=\"title\"
 						href=\"" . htmlspecialchars($line["link"]) . "\"
 						onclick=\"\">" .
 						truncate_string($line["title"], 200);
@@ -473,19 +481,22 @@ class Feeds extends Handler_Protected {
 
 					$reply['content'] .= $labels_str;
 
-					if (!get_pref($this->link, 'VFEED_GROUP_BY_FEED') &&
-						defined('_SHOW_FEED_TITLE_IN_VFEEDS')) {
+					$reply['content'] .= "</div>";
+
+					$reply['content'] .= "<span class=\"hlUpdated\">";
+
+					if (!get_pref($this->link, 'VFEED_GROUP_BY_FEED')) {
 						if (@$line["feed_title"]) {
-							$reply['content'] .= "<span class=\"hlFeed\">
-								(<a href=\"#\" onclick=\"viewfeed($feed_id)\">".
-								$line["feed_title"]."</a>)
-							</span>";
+							$reply['content'] .= "<div class=\"hlFeed\">
+								<a href=\"#\" onclick=\"viewfeed($feed_id)\">".
+								truncate_string($line["feed_title"],30)."</a>
+							</div>";
 						}
 					}
 
-					$reply['content'] .= "</div>";
+					$reply['content'] .= "<span title='$date_entered_fmt'>$updated_fmt</span>
+						</span>";
 
-					$reply['content'] .= "<span class=\"hlUpdated\">$updated_fmt</span>";
 					$reply['content'] .= "<div class=\"hlRight\">";
 
 					$reply['content'] .= $score_pic;
@@ -503,6 +514,16 @@ class Feeds extends Handler_Protected {
 
 				} else {
 
+					$line["tags"] = get_article_tags($this->link, $id, $_SESSION["uid"], $line["tag_cache"]);
+					unset($line["tag_cache"]);
+
+					$line["content"] = sanitize($this->link, $line["content_preview"],
+							sql_bool_to_bool($line['hide_images']), false, $entry_site_url);
+
+					foreach ($pluginhost->get_hooks($pluginhost::HOOK_RENDER_ARTICLE_CDM) as $p) {
+						$line = $p->hook_render_article_cdm($line);
+					}
+
 					if (get_pref($this->link, 'VFEED_GROUP_BY_FEED') && $line["feed_title"]) {
 						if ($feed_id != $vgroup_last_feed) {
 
@@ -511,7 +532,7 @@ class Feeds extends Handler_Protected {
 
 							$cur_feed_title = htmlspecialchars($cur_feed_title);
 
-							$vf_catchup_link = "(<a onclick='javascript:catchupFeedInGroup($feed_id);' href='#'>".__('mark as read')."</a>)";
+							$vf_catchup_link = "(<a class='catchup' onclick='javascript:catchupFeedInGroup($feed_id);' href='#'>".__('mark as read')."</a>)";
 
 							$has_feed_icon = feed_has_icon($feed_id);
 
@@ -523,55 +544,46 @@ class Feeds extends Handler_Protected {
 
 							$reply['content'] .= "<div class='cdmFeedTitle'>".
 								"<div style=\"float : right\">$feed_icon_img</div>".
-								"<a href=\"#\" onclick=\"viewfeed($feed_id)\">".
+								"<a href=\"#\" class='title' onclick=\"viewfeed($feed_id)\">".
 								$line["feed_title"]."</a> $vf_catchup_link</div>";
 						}
 					}
 
-					$expand_cdm = get_pref($this->link, 'CDM_EXPANDED');
-
-					$mouseover_attrs = "onmouseover='postMouseIn($id)'
+					$mouseover_attrs = "onmouseover='postMouseIn(event, $id)'
 						onmouseout='postMouseOut($id)'";
 
-					$reply['content'] .= "<div class=\"$class\" $label_row_style
+					$expanded_class = $expand_cdm ? "expanded" : "expandable";
+
+					$reply['content'] .= "<div class=\"cdm $expanded_class $class\"
 						id=\"RROW-$id\" $mouseover_attrs'>";
 
 					$reply['content'] .= "<div class=\"cdmHeader\">";
 
-					$reply['content'] .= "<div>";
+					$reply['content'] .= "<div style=\"vertical-align : middle\">";
 
-					$reply['content'] .= "<input type=\"checkbox\" onclick=\"toggleSelectRowById(this,
-							'RROW-$id')\" id=\"RCHK-$id\"/>";
+					$reply['content'] .= "<input dojoType=\"dijit.form.CheckBox\"
+							type=\"checkbox\" onclick=\"toggleSelectRow2(this, false, true)\"
+							id=\"RCHK-$id\">";
 
 					$reply['content'] .= "$marked_pic";
 					$reply['content'] .= "$published_pic";
 
 					$reply['content'] .= "</div>";
 
-					$reply['content'] .= "<div id=\"PTITLE-FULL-$id\" style=\"display : none\">" .
-						htmlspecialchars(strip_tags($line['title'])) . "</div>";
-
 					$reply['content'] .= "<span id=\"RTITLE-$id\"
 						onclick=\"return cdmClicked(event, $id);\"
 						class=\"titleWrap$hlc_suffix\">
 						<a class=\"title\"
-						title=\"".htmlspecialchars($line['title'])."\"
 						target=\"_blank\" href=\"".
 						htmlspecialchars($line["link"])."\">".
 						$line["title"] .
-						" $entry_author</a>";
+						" <span class=\"author\">$entry_author</span></a>";
 
 					$reply['content'] .= $labels_str;
 
-					if (!get_pref($this->link, 'VFEED_GROUP_BY_FEED') &&
-						defined('_SHOW_FEED_TITLE_IN_VFEEDS')) {
-						if (@$line["feed_title"]) {
-							$reply['content'] .= "<span class=\"hlFeed\">
-								(<a href=\"#\" onclick=\"viewfeed($feed_id)\">".
-								$line["feed_title"]."</a>)
-							</span>";
-						}
-					}
+					$reply['content'] .= "<span class='collapseBtn' style='display : none'>
+						<img src=\"images/collapse.png\" onclick=\"cdmCollapseArticle(event, $id)\"
+						title=\"".__("Collapse article")."\"/></span>";
 
 					if (!$expand_cdm)
 						$content_hidden = "style=\"display : none\"";
@@ -580,11 +592,21 @@ class Feeds extends Handler_Protected {
 
 					$reply['content'] .= "<span $excerpt_hidden
 						id=\"CEXC-$id\" class=\"cdmExcerpt\"> - $content_preview</span>";
-
 					$reply['content'] .= "</span>";
 
-					$reply['content'] .= "<div>";
-					$reply['content'] .= "<span class='updated'>$updated_fmt</span>";
+					if (!get_pref($this->link, 'VFEED_GROUP_BY_FEED')) {
+						if (@$line["feed_title"]) {
+							$reply['content'] .= "<div class=\"hlFeed\">
+								<a href=\"#\" onclick=\"viewfeed($feed_id)\">".
+								truncate_string($line["feed_title"],30)."</a>
+							</div>";
+						}
+					}
+
+					$reply['content'] .= "<span class='updated' title='$date_entered_fmt'>
+						$updated_fmt</span>";
+
+					$reply['content'] .= "<div style=\"vertical-align : middle\">";
 					$reply['content'] .= "$score_pic";
 
 					if (!get_pref($this->link, "VFEED_GROUP_BY_FEED") && $line["feed_title"]) {
@@ -592,7 +614,6 @@ class Feeds extends Handler_Protected {
 							title=\"".htmlspecialchars($line["feed_title"])."\"
 							onclick=\"viewfeed($feed_id)\">$feed_icon_img</span>";
 					}
-					$reply['content'] .= "<div class=\"updPic\">$update_pic</div>";
 					$reply['content'] .= "</div>";
 
 					$reply['content'] .= "</div>";
@@ -601,6 +622,13 @@ class Feeds extends Handler_Protected {
 						onclick=\"return cdmClicked(event, $id);\"
 						id=\"CICD-$id\">";
 
+					$reply['content'] .= "<div id=\"POSTNOTE-$id\">";
+					if ($line['note']) {
+						$reply['content'] .= format_article_note($id, $line['note']);
+					}
+					$reply['content'] .= "</div>";
+
+
 					$reply['content'] .= "<div class=\"cdmContentInner\">";
 
 			if ($line["orig_feed_id"]) {
@@ -624,56 +652,36 @@ class Feeds extends Handler_Protected {
 							$reply['content'] .= "&nbsp;";
 
 							$reply['content'] .= "<a target='_blank' href='" . htmlspecialchars($tmp_line['feed_url']) . "'>";
-							$reply['content'] .= "<img title='".__('Feed URL')."'class='tinyFeedIcon' src='images/pub_set.png'></a>";
+							$reply['content'] .= "<img title='".__('Feed URL')."'class='tinyFeedIcon' src='images/pub_unset.svg'></a>";
 
 							$reply['content'] .= "</div>";
 						}
 					}
 
-					$feed_site_url = $line["site_url"];
-
-					if ($cache_content && $line["cached_content"] != "") {
-						$line["content_preview"] =& $line["cached_content"];
-					}
-
-					$article_content = sanitize($this->link, $line["content_preview"],
-							false, false, $feed_site_url);
-
-					$reply['content'] .= "<div id=\"POSTNOTE-$id\">";
-					if ($line['note']) {
-						$reply['content'] .= format_article_note($id, $line['note']);
-					}
-					$reply['content'] .= "</div>";
-
 					$reply['content'] .= "<span id=\"CWRAP-$id\">";
-					$reply['content'] .= $article_content;
+
+//					if (!$expand_cdm) {
+						$reply['content'] .= "<span id=\"CENCW-$id\" style=\"display : none\">";
+						$reply['content'] .= htmlspecialchars($line["content"]);
+						$reply['content'] .= "</span.";
+
+//					} else {
+//						$reply['content'] .= $line["content"];
+//					}
+
 					$reply['content'] .= "</span>";
 
-/*					$tmp_result = db_query($this->link, "SELECT always_display_enclosures FROM
-						ttrss_feeds WHERE id = ".
-						(($line['feed_id'] == null) ? $line['orig_feed_id'] :
-							$line['feed_id'])." AND owner_uid = ".$_SESSION["uid"]);
-
-					$always_display_enclosures = sql_bool_to_bool(db_fetch_result($tmp_result,
-						0, "always_display_enclosures")); */
-
 					$always_display_enclosures = sql_bool_to_bool($line["always_display_enclosures"]);
 
-					$reply['content'] .= format_article_enclosures($this->link, $id, $always_display_enclosures,
-						$article_content);
+					$reply['content'] .= format_article_enclosures($this->link, $id, $always_display_enclosures, $line["content"], sql_bool_to_bool($line["hide_images"]));
 
 					$reply['content'] .= "</div>";
 
 					$reply['content'] .= "<div class=\"cdmFooter\">";
 
-					$tag_cache = $line["tag_cache"];
+					$tags_str = format_tags_string($line["tags"], $id);
 
-					$tags_str = format_tags_string(
-						get_article_tags($this->link, $id, $_SESSION["uid"], $tag_cache),
-						$id);
-
-					$reply['content'] .= "<img src='".theme_image($this->link,
-							'images/tag.png')."' alt='Tags' title='Tags'>
+					$reply['content'] .= "<img src='images/tag.png' alt='Tags' title='Tags'>
 						<span id=\"ATSTR-$id\">$tags_str</span>
 						<a title=\"".__('Edit tags for this article')."\"
 						href=\"#\" onclick=\"editArticleTags($id, $feed_id, true)\">(+)</a>";
@@ -698,23 +706,10 @@ class Feeds extends Handler_Protected {
 
 					$reply['content'] .= "<div style=\"float : right\">";
 
-					$reply['content'] .= "<img src=\"images/art-zoom.png\"
-						onclick=\"zoomToArticle(event, $id)\"
-						style=\"cursor : pointer\"
-						alt='Zoom'
-						title='".__('Open article in new tab')."'>";
-
-					//$note_escaped = htmlspecialchars($line['note'], ENT_QUOTES);
-
-					foreach ($button_plugins as $p) {
-						$reply['content'] .= $p->render($id, $line);
+					foreach ($pluginhost->get_hooks($pluginhost::HOOK_ARTICLE_BUTTON) as $p) {
+						$reply['content'] .= $p->hook_article_button($line);
 					}
 
-					$reply['content'] .= "<img src=\"images/digest_checkbox.png\"
-						style=\"cursor : pointer\" style=\"cursor : pointer\"
-						onclick=\"dismissArticle($id)\"
-						title='".__('Close article')."'>";
-
 					$reply['content'] .= "</div>";
 					$reply['content'] .= "</div>";
 
@@ -743,8 +738,8 @@ class Feeds extends Handler_Protected {
 					$message = __("No starred articles found to display.");
 					break;
 				default:
-					if ($feed < -10) {
-						$message = __("No articles found to display. You can assign articles to labels manually (see the Actions menu above) or use a filter.");
+					if ($feed < LABEL_BASE_INDEX) {
+						$message = __("No articles found to display. You can assign articles to labels manually from article header context menu (applies to all selected articles) or use a filter.");
 					} else {
 						$message = __("No articles found to display.");
 					}
@@ -753,7 +748,7 @@ class Feeds extends Handler_Protected {
 			if (!$offset && $message) {
 				$reply['content'] .= "<div class='whiteBox'>$message";
 
-				$reply['content'] .= "<p class=\"small\"><span class=\"insensitive\">";
+				$reply['content'] .= "<p><span class=\"insensitive\">";
 
 				$result = db_query($this->link, "SELECT ".SUBSTRING_FOR_DATE."(MAX(last_updated), 1, 19) AS last_updated FROM ttrss_feeds
 					WHERE owner_uid = " . $_SESSION['uid']);
@@ -785,34 +780,28 @@ class Feeds extends Handler_Protected {
 
 	function catchupAll() {
 		db_query($this->link, "UPDATE ttrss_user_entries SET
-						last_read = NOW(),unread = false WHERE owner_uid = " . $_SESSION["uid"]);
+						last_read = NOW(), unread = false WHERE unread = true AND owner_uid = " . $_SESSION["uid"]);
 		ccache_zero_all($this->link, $_SESSION["uid"]);
 	}
 
-	function collapse() {
-		$cat_id = db_escape_string($_REQUEST["cid"]);
-		$mode = (int) db_escape_string($_REQUEST['mode']);
-		toggle_collapse_cat($this->link, $cat_id, $mode);
-	}
-
 	function view() {
-		$timing_info = getmicrotime();
+		$timing_info = microtime(true);
 
 		$reply = array();
 
 		if ($_REQUEST["debug"]) $timing_info = print_checkpoint("0", $timing_info);
 
-		$omode = db_escape_string($_REQUEST["omode"]);
+		$omode = db_escape_string($this->link, $_REQUEST["omode"]);
 
-		$feed = db_escape_string($_REQUEST["feed"]);
-		$method = db_escape_string($_REQUEST["m"]);
-		$view_mode = db_escape_string($_REQUEST["view_mode"]);
-		$limit = (int) get_pref($this->link, "DEFAULT_ARTICLE_LIMIT");
+		$feed = db_escape_string($this->link, $_REQUEST["feed"]);
+		$method = db_escape_string($this->link, $_REQUEST["m"]);
+		$view_mode = db_escape_string($this->link, $_REQUEST["view_mode"]);
+		$limit = 30;
 		@$cat_view = $_REQUEST["cat"] == "true";
-		@$next_unread_feed = db_escape_string($_REQUEST["nuf"]);
-		@$offset = db_escape_string($_REQUEST["skip"]);
-		@$vgroup_last_feed = db_escape_string($_REQUEST["vgrlf"]);
-		$order_by = db_escape_string($_REQUEST["order_by"]);
+		@$next_unread_feed = db_escape_string($this->link, $_REQUEST["nuf"]);
+		@$offset = db_escape_string($this->link, $_REQUEST["skip"]);
+		@$vgroup_last_feed = db_escape_string($this->link, $_REQUEST["vgrlf"]);
+		$order_by = db_escape_string($this->link, $_REQUEST["order_by"]);
 
 		if (is_numeric($feed)) $feed = (int) $feed;
 
@@ -820,14 +809,14 @@ class Feeds extends Handler_Protected {
 		 * when there's nothing to load - e.g. no stuff in fresh feed */
 
 		if ($feed == -5) {
-			print json_encode(generate_dashboard_feed($this->link));
+			print json_encode($this->generate_dashboard_feed($this->link));
 			return;
 		}
 
 		$result = false;
 
-		if ($feed < -10) {
-			$label_feed = -11-$feed;
+		if ($feed < LABEL_BASE_INDEX) {
+			$label_feed = feed_to_label_id($feed);
 			$result = db_query($this->link, "SELECT id FROM ttrss_labels2 WHERE
 							id = '$label_feed' AND owner_uid = " . $_SESSION['uid']);
 		} else if (!$cat_view && is_numeric($feed) && $feed > 0) {
@@ -839,7 +828,7 @@ class Feeds extends Handler_Protected {
 		}
 
 		if ($result && db_num_rows($result) == 0) {
-			print json_encode(generate_error_feed($this->link, __("Feed not found.")));
+			print json_encode($this->generate_error_feed($this->link, __("Feed not found.")));
 			return;
 		}
 
@@ -851,9 +840,15 @@ class Feeds extends Handler_Protected {
 		}
 
 		set_pref($this->link, "_DEFAULT_VIEW_MODE", $view_mode);
-		set_pref($this->link, "_DEFAULT_VIEW_LIMIT", $limit);
 		set_pref($this->link, "_DEFAULT_VIEW_ORDER_BY", $order_by);
 
+		/* bump login timestamp if needed */
+		if (time() - $_SESSION["last_login_update"] > 3600) {
+			db_query($this->link, "UPDATE ttrss_users SET last_login = NOW() WHERE id = " .
+				$_SESSION["uid"]);
+			$_SESSION["last_login_update"] = time();
+		}
+
 		if (!$cat_view && is_numeric($feed) && $feed > 0) {
 			db_query($this->link, "UPDATE ttrss_feeds SET last_viewed = NOW()
 							WHERE id = '$feed' AND owner_uid = ".$_SESSION["uid"]);
@@ -870,35 +865,12 @@ class Feeds extends Handler_Protected {
 
 		$override_order = false;
 
-		if (get_pref($this->link, "SORT_HEADLINES_BY_FEED_DATE", $owner_uid)) {
-			$date_sort_field = "updated";
-		} else {
-			$date_sort_field = "date_entered";
-		}
-
 		switch ($order_by) {
-			case "date":
-				if (get_pref($this->link, 'REVERSE_HEADLINES', $owner_uid)) {
-					$override_order = "$date_sort_field";
-				} else {
-					$override_order = "$date_sort_field DESC";
-				}
+			case "date_reverse":
+				$override_order = "date_entered, updated";
 				break;
-
-			case "title":
-				if (get_pref($this->link, 'REVERSE_HEADLINES', $owner_uid)) {
-					$override_order = "title DESC, $date_sort_field";
-				} else {
-					$override_order = "title, $date_sort_field DESC";
-				}
-				break;
-
-			case "score":
-				if (get_pref($this->link, 'REVERSE_HEADLINES', $owner_uid)) {
-					$override_order = "score, $date_sort_field";
-				} else {
-					$override_order = "score DESC, $date_sort_field DESC";
-				}
+			case "feed_dates":
+				$override_order = "updated DESC";
 				break;
 		}
 
@@ -908,7 +880,7 @@ class Feeds extends Handler_Protected {
 			$view_mode, $limit, $cat_view, $next_unread_feed, $offset,
 			$vgroup_last_feed, $override_order, true);
 
-		$topmost_article_ids = $ret[0];
+		//$topmost_article_ids = $ret[0];
 		$headlines_count = $ret[1];
 		$returned_feed = $ret[2];
 		$disable_cache = $ret[3];
@@ -923,18 +895,6 @@ class Feeds extends Handler_Protected {
 						"vgroup_last_feed" => $vgroup_last_feed,
 						"disable_cache" => (bool) $disable_cache);
 
-		if ($_REQUEST["debug"]) $timing_info = print_checkpoint("20", $timing_info);
-
-		if (is_array($topmost_article_ids) && !get_pref($this->link, 'COMBINED_DISPLAY_MODE') && !$_SESSION["bw_limit"]) {
-			$articles = array();
-
-			foreach ($topmost_article_ids as $id) {
-				array_push($articles, format_article($this->link, $id, false));
-			}
-
-			$reply['articles'] = $articles;
-		}
-
 		if ($_REQUEST["debug"]) $timing_info = print_checkpoint("30", $timing_info);
 
 		$reply['runtime-info'] = make_runtime_info($this->link);
@@ -942,5 +902,245 @@ class Feeds extends Handler_Protected {
 		print json_encode($reply);
 
 	}
+
+	private function generate_dashboard_feed($link) {
+		$reply = array();
+
+		$reply['headlines']['id'] = -5;
+		$reply['headlines']['is_cat'] = false;
+
+		$reply['headlines']['toolbar'] = '';
+		$reply['headlines']['content'] = "<div class='whiteBox'>".__('No feed selected.');
+
+		$reply['headlines']['content'] .= "<p><span class=\"insensitive\">";
+
+		$result = db_query($link, "SELECT ".SUBSTRING_FOR_DATE."(MAX(last_updated), 1, 19) AS last_updated FROM ttrss_feeds
+			WHERE owner_uid = " . $_SESSION['uid']);
+
+		$last_updated = db_fetch_result($result, 0, "last_updated");
+		$last_updated = make_local_datetime($link, $last_updated, false);
+
+		$reply['headlines']['content'] .= sprintf(__("Feeds last updated at %s"), $last_updated);
+
+		$result = db_query($link, "SELECT COUNT(id) AS num_errors
+			FROM ttrss_feeds WHERE last_error != '' AND owner_uid = ".$_SESSION["uid"]);
+
+		$num_errors = db_fetch_result($result, 0, "num_errors");
+
+		if ($num_errors > 0) {
+			$reply['headlines']['content'] .= "<br/>";
+			$reply['headlines']['content'] .= "<a class=\"insensitive\" href=\"#\" onclick=\"showFeedsWithErrors()\">".
+				__('Some feeds have update errors (click for details)')."</a>";
+		}
+		$reply['headlines']['content'] .= "</span></p>";
+
+		$reply['headlines-info'] = array("count" => 0,
+			"vgroup_last_feed" => '',
+			"unread" => 0,
+			"disable_cache" => true);
+
+		return $reply;
+	}
+
+	private function generate_error_feed($link, $error) {
+		$reply = array();
+
+		$reply['headlines']['id'] = -6;
+		$reply['headlines']['is_cat'] = false;
+
+		$reply['headlines']['toolbar'] = '';
+		$reply['headlines']['content'] = "<div class='whiteBox'>". $error . "</div>";
+
+		$reply['headlines-info'] = array("count" => 0,
+			"vgroup_last_feed" => '',
+			"unread" => 0,
+			"disable_cache" => true);
+
+		return $reply;
+	}
+
+	function quickAddFeed() {
+		print "<input dojoType=\"dijit.form.TextBox\" style=\"display : none\" name=\"op\" value=\"rpc\">";
+		print "<input dojoType=\"dijit.form.TextBox\" style=\"display : none\" name=\"method\" value=\"addfeed\">";
+
+		print "<div class=\"dlgSec\">".__("Feed or site URL")."</div>";
+		print "<div class=\"dlgSecCont\">";
+
+		print "<div style='float : right'>
+			<img style='display : none'
+				id='feed_add_spinner' src='images/indicator_white.gif'></div>";
+
+		print "<input style=\"font-size : 16px; width : 20em;\"
+			placeHolder=\"".__("Feed or site URL")."\"
+			dojoType=\"dijit.form.ValidationTextBox\" required=\"1\" name=\"feed\" id=\"feedDlg_feedUrl\">";
+
+		print "<hr/>";
+
+		if (get_pref($this->link, 'ENABLE_FEED_CATS')) {
+			print __('Place in category:') . " ";
+			print_feed_cat_select($this->link, "cat", false, 'dojoType="dijit.form.Select"');
+		}
+
+		print "</div>";
+
+		print '<div id="feedDlg_feedsContainer" style="display : none">
+
+				<div class="dlgSec">' . __('Available feeds') . '</div>
+				<div class="dlgSecCont">'.
+				'<select id="feedDlg_feedContainerSelect"
+					dojoType="dijit.form.Select" size="3">
+					<script type="dojo/method" event="onChange" args="value">
+						dijit.byId("feedDlg_feedUrl").attr("value", value);
+					</script>
+				</select>'.
+				'</div></div>';
+
+		print "<div id='feedDlg_loginContainer' style='display : none'>
+
+				<div class=\"dlgSec\">".__("Authentication")."</div>
+				<div class=\"dlgSecCont\">".
+
+				" <input dojoType=\"dijit.form.TextBox\" name='login'\"
+					placeHolder=\"".__("Login")."\"
+					style=\"width : 10em;\"> ".
+				" <input
+					placeHolder=\"".__("Password")."\"
+					dojoType=\"dijit.form.TextBox\" type='password'
+					style=\"width : 10em;\" name='pass'\">
+			</div></div>";
+
+
+		print "<div style=\"clear : both\">
+			<input type=\"checkbox\" name=\"need_auth\" dojoType=\"dijit.form.CheckBox\" id=\"feedDlg_loginCheck\"
+					onclick='checkboxToggleElement(this, \"feedDlg_loginContainer\")'>
+				<label for=\"feedDlg_loginCheck\">".
+				__('This feed requires authentication.')."</div>";
+
+		print "</form>";
+
+		print "<div class=\"dlgButtons\">
+			<button dojoType=\"dijit.form.Button\" onclick=\"return dijit.byId('feedAddDlg').execute()\">".__('Subscribe')."</button>";
+
+		if (!(defined('_DISABLE_FEED_BROWSER') && _DISABLE_FEED_BROWSER)) {
+			print "<button dojoType=\"dijit.form.Button\" onclick=\"return feedBrowser()\">".__('More feeds')."</button>";
+		}
+
+		print "<button dojoType=\"dijit.form.Button\" onclick=\"return dijit.byId('feedAddDlg').hide()\">".__('Cancel')."</button>
+			</div>";
+
+		//return;
+	}
+
+	function feedBrowser() {
+		if (defined('_DISABLE_FEED_BROWSER') && _DISABLE_FEED_BROWSER) return;
+
+		$browser_search = db_escape_string($this->link, $_REQUEST["search"]);
+
+		print "<input dojoType=\"dijit.form.TextBox\" style=\"display : none\" name=\"op\" value=\"rpc\">";
+		print "<input dojoType=\"dijit.form.TextBox\" style=\"display : none\" name=\"method\" value=\"updateFeedBrowser\">";
+
+		print "<div dojoType=\"dijit.Toolbar\">
+			<div style='float : right'>
+			<img style='display : none'
+				id='feed_browser_spinner' src='images/indicator_white.gif'>
+			<input name=\"search\" dojoType=\"dijit.form.TextBox\" size=\"20\" type=\"search\"
+				onchange=\"dijit.byId('feedBrowserDlg').update()\" value=\"$browser_search\">
+			<button dojoType=\"dijit.form.Button\" onclick=\"dijit.byId('feedBrowserDlg').update()\">".__('Search')."</button>
+		</div>";
+
+		print " <select name=\"mode\" dojoType=\"dijit.form.Select\" onchange=\"dijit.byId('feedBrowserDlg').update()\">
+			<option value='1'>" . __('Popular feeds') . "</option>
+			<option value='2'>" . __('Feed archive') . "</option>
+			</select> ";
+
+		print __("limit:");
+
+		print " <select dojoType=\"dijit.form.Select\" name=\"limit\" onchange=\"dijit.byId('feedBrowserDlg').update()\">";
+
+		foreach (array(25, 50, 100, 200) as $l) {
+			$issel = ($l == $limit) ? "selected=\"1\"" : "";
+			print "<option $issel value=\"$l\">$l</option>";
+		}
+
+		print "</select> ";
+
+		print "</div>";
+
+		$owner_uid = $_SESSION["uid"];
+
+		require_once "feedbrowser.php";
+
+		print "<ul class='browseFeedList' id='browseFeedList'>";
+		print make_feed_browser($this->link, $search, 25);
+		print "</ul>";
+
+		print "<div align='center'>
+			<button dojoType=\"dijit.form.Button\" onclick=\"dijit.byId('feedBrowserDlg').execute()\">".__('Subscribe')."</button>
+			<button dojoType=\"dijit.form.Button\" style='display : none' id='feed_archive_remove' onclick=\"dijit.byId('feedBrowserDlg').removeFromArchive()\">".__('Remove')."</button>
+			<button dojoType=\"dijit.form.Button\" onclick=\"dijit.byId('feedBrowserDlg').hide()\" >".__('Cancel')."</button></div>";
+
+	}
+
+	function search() {
+		$this->params = explode(":", db_escape_string($this->link, $_REQUEST["param"]), 2);
+
+		$active_feed_id = sprintf("%d", $this->params[0]);
+		$is_cat = $this->params[1] != "false";
+
+		print "<div class=\"dlgSec\">".__('Look for')."</div>";
+
+		print "<div class=\"dlgSecCont\">";
+
+		print "<input dojoType=\"dijit.form.ValidationTextBox\"
+			style=\"font-size : 16px; width : 20em;\"
+			required=\"1\" name=\"query\" type=\"search\" value=''>";
+
+		print "<hr/>".__('Limit search to:')." ";
+
+		print "<select name=\"search_mode\" dojoType=\"dijit.form.Select\">
+			<option value=\"all_feeds\">".__('All feeds')."</option>";
+
+		$feed_title = getFeedTitle($this->link, $active_feed_id);
+
+		if (!$is_cat) {
+			$feed_cat_title = getFeedCatTitle($this->link, $active_feed_id);
+		} else {
+			$feed_cat_title = getCategoryTitle($this->link, $active_feed_id);
+		}
+
+		if ($active_feed_id && !$is_cat) {
+			print "<option selected=\"1\" value=\"this_feed\">$feed_title</option>";
+		} else {
+			print "<option disabled=\"1\" value=\"false\">".__('This feed')."</option>";
+		}
+
+		if ($is_cat) {
+		  	$cat_preselected = "selected=\"1\"";
+		}
+
+		if (get_pref($this->link, 'ENABLE_FEED_CATS') && ($active_feed_id > 0 || $is_cat)) {
+			print "<option $cat_preselected value=\"this_cat\">$feed_cat_title</option>";
+		} else {
+			//print "<option disabled>".__('This category')."</option>";
+		}
+
+		print "</select>";
+
+		print "</div>";
+
+		print "<div class=\"dlgButtons\">";
+
+		if (!SPHINX_ENABLED) {
+			print "<div style=\"float : left\">
+				<a class=\"visibleLink\" target=\"_blank\" href=\"http://tt-rss.org/wiki/SearchSyntax\">Search syntax</a>
+				</div>";
+		}
+
+		print "<button dojoType=\"dijit.form.Button\" onclick=\"dijit.byId('searchDlg').execute()\">".__('Search')."</button>
+		<button dojoType=\"dijit.form.Button\" onclick=\"dijit.byId('searchDlg').hide()\">".__('Cancel')."</button>
+		</div>";
+	}
+
+
 }
 ?>
