@@ -2,7 +2,7 @@
 class Handler_Public extends Handler {
 
 	private function generate_syndicated_feed($owner_uid, $feed, $is_cat,
-		$limit, $offset, $search, $search_mode, $match_on,
+		$limit, $offset, $search, $search_mode,
 		$view_mode = false, $format = 'atom') {
 
 		require_once "lib/MiniTemplator.class.php";
@@ -12,20 +12,40 @@ class Handler_Public extends Handler {
 			"padding : 5px; border-style : dashed; border-color : #e7d796;".
 			"margin-bottom : 1em; color : #9a8c59;";
 
-		if (!$limit) $limit = 100;
+		if (!$limit) $limit = 60;
 
-		if (get_pref($this->link, "SORT_HEADLINES_BY_FEED_DATE", $owner_uid)) {
-			$date_sort_field = "updated";
-		} else {
-			$date_sort_field = "date_entered";
-		}
+		$date_sort_field = "date_entered DESC, updated DESC";
 
 		if ($feed == -2)
-			$date_sort_field = "last_read";
+			$date_sort_field = "last_published DESC";
+		else if ($feed == -1)
+			$date_sort_field = "last_marked DESC";
+
+		$qfh_ret = queryFeedHeadlines($this->link, $feed,
+			1, $view_mode, $is_cat, $search, $search_mode,
+			$date_sort_field, $offset, $owner_uid,
+			false, 0, false, true);
+
+		$result = $qfh_ret[0];
+
+		if (db_num_rows($result) != 0) {
+			$ts = strtotime(db_fetch_result($result, 0, "date_entered"));
+
+			if (isset($_SERVER['HTTP_IF_MODIFIED_SINCE']) &&
+					strtotime($_SERVER['HTTP_IF_MODIFIED_SINCE']) >= $ts) {
+		      header('HTTP/1.0 304 Not Modified');
+		      return;
+			}
+
+			$last_modified = gmdate("D, d M Y H:i:s", $ts) . " GMT";
+			header("Last-Modified: $last_modified", true);
+		}
 
 		$qfh_ret = queryFeedHeadlines($this->link, $feed,
 			$limit, $view_mode, $is_cat, $search, $search_mode,
-			$match_on, "$date_sort_field DESC", $offset, $owner_uid);
+			$date_sort_field, $offset, $owner_uid,
+			false, 0, false, true);
+
 
 		$result = $qfh_ret[0];
 		$feed_title = htmlspecialchars($qfh_ret[1]);
@@ -54,7 +74,8 @@ class Handler_Public extends Handler {
 
 			$tpl->setVariable('SELF_URL', htmlspecialchars(get_self_url_prefix()), true);
 
-	 		while ($line = db_fetch_assoc($result)) {
+			while ($line = db_fetch_assoc($result)) {
+
 				$tpl->setVariable('ARTICLE_ID', htmlspecialchars($line['link']), true);
 				$tpl->setVariable('ARTICLE_LINK', htmlspecialchars($line['link']), true);
 				$tpl->setVariable('ARTICLE_TITLE', htmlspecialchars($line['title']), true);
@@ -66,6 +87,7 @@ class Handler_Public extends Handler {
 				if ($line['note']) {
 					$content = "<div style=\"$note_style\">Article note: " . $line['note'] . "</div>" .
 						$content;
+					$tpl->setVariable('ARTICLE_NOTE', htmlspecialchars($line['note']), true);
 				}
 
 				$tpl->setVariable('ARTICLE_CONTENT', $content, true);
@@ -106,7 +128,11 @@ class Handler_Public extends Handler {
 			$tpl->addBlock('feed');
 			$tpl->generateOutputToString($tmp);
 
-			header("Content-Type: text/xml; charset=utf-8");
+			if (@!$_REQUEST["noxml"]) {
+				header("Content-Type: text/xml; charset=utf-8");
+			} else {
+				header("Content-Type: text/plain; charset=utf-8");
+			}
 
 			print $tmp;
 		} else if ($format == 'json') {
@@ -165,8 +191,7 @@ class Handler_Public extends Handler {
 				array_push($feed['articles'], $article);
 			}
 
-			header("Content-Type: text/plain; charset=utf-8");
-
+			header("Content-Type: text/json; charset=utf-8");
 			print json_encode($feed);
 
 		} else {
@@ -176,7 +201,7 @@ class Handler_Public extends Handler {
 	}
 
 	function getUnread() {
-		$login = db_escape_string($_REQUEST["login"]);
+		$login = db_escape_string($this->link, $_REQUEST["login"]);
 		$fresh = $_REQUEST["fresh"] == "1";
 
 		$result = db_query($this->link, "SELECT id FROM ttrss_users WHERE login = '$login'");
@@ -198,7 +223,7 @@ class Handler_Public extends Handler {
 	}
 
 	function getProfiles() {
-		$login = db_escape_string($_REQUEST["login"]);
+		$login = db_escape_string($this->link, $_REQUEST["login"]);
 
 		$result = db_query($this->link, "SELECT * FROM ttrss_settings_profiles,ttrss_users
 			WHERE ttrss_users.id = ttrss_settings_profiles.owner_uid AND login = '$login' ORDER BY title");
@@ -218,9 +243,9 @@ class Handler_Public extends Handler {
 	}
 
 	function pubsub() {
-		$mode = db_escape_string($_REQUEST['hub_mode']);
-		$feed_id = (int) db_escape_string($_REQUEST['id']);
-		$feed_url = db_escape_string($_REQUEST['hub_topic']);
+		$mode = db_escape_string($this->link, $_REQUEST['hub_mode']);
+		$feed_id = (int) db_escape_string($this->link, $_REQUEST['id']);
+		$feed_url = db_escape_string($this->link, $_REQUEST['hub_topic']);
 
 		if (!PUBSUBHUBBUB_ENABLED) {
 			header('HTTP/1.0 404 Not Found');
@@ -280,38 +305,8 @@ class Handler_Public extends Handler {
 		header("Location: index.php");
 	}
 
-	function fbexport() {
-
-		$access_key = db_escape_string($_POST["key"]);
-
-		// TODO: rate limit checking using last_connected
-		$result = db_query($this->link, "SELECT id FROM ttrss_linked_instances
-			WHERE access_key = '$access_key'");
-
-		if (db_num_rows($result) == 1) {
-
-			$instance_id = db_fetch_result($result, 0, "id");
-
-			$result = db_query($this->link, "SELECT feed_url, site_url, title, subscribers
-				FROM ttrss_feedbrowser_cache ORDER BY subscribers DESC LIMIT 100");
-
-			$feeds = array();
-
-			while ($line = db_fetch_assoc($result)) {
-				array_push($feeds, $line);
-			}
-
-			db_query($this->link, "UPDATE ttrss_linked_instances SET
-				last_status_in = 1 WHERE id = '$instance_id'");
-
-			print json_encode(array("feeds" => $feeds));
-		} else {
-			print json_encode(array("error" => array("code" => 6)));
-		}
-	}
-
 	function share() {
-		$uuid = db_escape_string($_REQUEST["key"]);
+		$uuid = db_escape_string($this->link, $_REQUEST["key"]);
 
 		$result = db_query($this->link, "SELECT ref_id, owner_uid FROM ttrss_user_entries WHERE
 			uuid = '$uuid'");
@@ -333,18 +328,17 @@ class Handler_Public extends Handler {
 	}
 
 	function rss() {
-		$feed = db_escape_string($_REQUEST["id"]);
-		$key = db_escape_string($_REQUEST["key"]);
+		$feed = db_escape_string($this->link, $_REQUEST["id"]);
+		$key = db_escape_string($this->link, $_REQUEST["key"]);
 		$is_cat = $_REQUEST["is_cat"] != false;
-		$limit = (int)db_escape_string($_REQUEST["limit"]);
-		$offset = (int)db_escape_string($_REQUEST["offset"]);
+		$limit = (int)db_escape_string($this->link, $_REQUEST["limit"]);
+		$offset = (int)db_escape_string($this->link, $_REQUEST["offset"]);
 
-		$search = db_escape_string($_REQUEST["q"]);
-		$match_on = db_escape_string($_REQUEST["m"]);
-		$search_mode = db_escape_string($_REQUEST["smode"]);
-		$view_mode = db_escape_string($_REQUEST["view-mode"]);
+		$search = db_escape_string($this->link, $_REQUEST["q"]);
+		$search_mode = db_escape_string($this->link, $_REQUEST["smode"]);
+		$view_mode = db_escape_string($this->link, $_REQUEST["view-mode"]);
 
-		$format = db_escape_string($_REQUEST['format']);
+		$format = db_escape_string($this->link, $_REQUEST['format']);
 
 		if (!$format) $format = 'atom';
 
@@ -364,7 +358,7 @@ class Handler_Public extends Handler {
 
 		if ($owner_id) {
 			$this->generate_syndicated_feed($owner_id, $feed, $is_cat, $limit,
-				$offset, $search, $search_mode, $match_on, $view_mode, $format);
+				$offset, $search, $search_mode, $view_mode, $format);
 		} else {
 			header('HTTP/1.1 403 Forbidden');
 		}
@@ -374,6 +368,18 @@ class Handler_Public extends Handler {
 		include "rssfuncs.php";
 		// Update all feeds needing a update.
 		update_daemon_common($this->link, 0, true, false);
+
+		// Update feedbrowser
+		update_feedbrowser_cache($this->link);
+
+		// Purge orphans and cleanup tags
+		purge_orphans($this->link);
+
+		cleanup_tags($this->link, 14, 50000);
+
+		global $pluginhost;
+		$pluginhost->run_hooks($pluginhost::HOOK_UPDATE_TASK, "hook_update_task", $op);
+
 	}
 
 	function sharepopup() {
@@ -382,15 +388,13 @@ class Handler_Public extends Handler {
 		}
 
 		header('Content-Type: text/html; charset=utf-8');
-		print "<html>
-				<head>
-					<title>Tiny Tiny RSS</title>
-					<link rel=\"stylesheet\" type=\"text/css\" href=\"utility.css\">
-					<script type=\"text/javascript\" src=\"lib/prototype.js\"></script>
-					<script type=\"text/javascript\" src=\"lib/scriptaculous/scriptaculous.js?load=effects,dragdrop,controls\"></script>
-					<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"/>
-				</head>
-				<body id='sharepopup'>";
+		print "<html><head><title>Tiny Tiny RSS</title>";
+
+		print stylesheet_tag("utility.css");
+		print javascript_tag("lib/prototype.js");
+		print javascript_tag("lib/scriptaculous/scriptaculous.js?load=effects,dragdrop,controls");
+		print "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"/>
+			</head><body id='sharepopup'>";
 
 		$action = $_REQUEST["action"];
 
@@ -398,12 +402,12 @@ class Handler_Public extends Handler {
 
 			if ($action == 'share') {
 
-				$title = db_escape_string(strip_tags($_REQUEST["title"]));
-				$url = db_escape_string(strip_tags($_REQUEST["url"]));
-				$content = db_escape_string(strip_tags($_REQUEST["content"]));
-				$labels = db_escape_string(strip_tags($_REQUEST["labels"]));
+				$title = db_escape_string($this->link, strip_tags($_REQUEST["title"]));
+				$url = db_escape_string($this->link, strip_tags($_REQUEST["url"]));
+				$content = db_escape_string($this->link, strip_tags($_REQUEST["content"]));
+				$labels = db_escape_string($this->link, strip_tags($_REQUEST["labels"]));
 
-				create_published_article($this->link, $title, $url, $content, $labels,
+				Article::create_published_article($this->link, $title, $url, $content, $labels,
 					$_SESSION["uid"]);
 
 				print "<script type='text/javascript'>";
@@ -505,15 +509,22 @@ class Handler_Public extends Handler {
 	}
 
 	function login() {
-
 		$_SESSION["prefs_cache"] = array();
 
 		if (!SINGLE_USER_MODE) {
 
-			$login = db_escape_string($_POST["login"]);
+			$login = db_escape_string($this->link, $_POST["login"]);
 			$password = $_POST["password"];
 			$remember_me = $_POST["remember_me"];
 
+			if ($remember_me) {
+				session_set_cookie_params(SESSION_COOKIE_LIFETIME);
+			} else {
+				session_set_cookie_params(0);
+			}
+
+			@session_start();
+
 			if (authenticate_user($this->link, $login, $password)) {
 				$_POST["password"] = "";
 
@@ -523,7 +534,7 @@ class Handler_Public extends Handler {
 
 				if ($_POST["profile"]) {
 
-					$profile = db_escape_string($_POST["profile"]);
+					$profile = db_escape_string($this->link, $_POST["profile"]);
 
 					$result = db_query($this->link, "SELECT id FROM ttrss_settings_profiles
 						WHERE id = '$profile' AND owner_uid = " . $_SESSION["uid"]);
@@ -552,7 +563,7 @@ class Handler_Public extends Handler {
 
 		if ($_SESSION["uid"]) {
 
-			$feed_url = db_escape_string(trim($_REQUEST["feed_url"]));
+			$feed_url = db_escape_string($this->link, trim($_REQUEST["feed_url"]));
 
 			header('Content-Type: text/html; charset=utf-8');
 			print "<html>
@@ -562,9 +573,9 @@ class Handler_Public extends Handler {
 					<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"/>
 				</head>
 				<body>
-				<img class=\"floatingLogo\" src=\"images/logo_wide.png\"
+				<img class=\"floatingLogo\" src=\"images/logo_small.png\"
 			  		alt=\"Tiny Tiny RSS\"/>
-					<h1>".__("Subscribe to feed...")."</h1>";
+					<h1>".__("Subscribe to feed...")."</h1><div class='content'>";
 
 			$rc = subscribe_to_feed($this->link, $feed_url);
 
@@ -637,7 +648,7 @@ class Handler_Public extends Handler {
 				<input type=\"submit\" value=\"".__("Return to Tiny Tiny RSS")."\">
 				</form></p>";
 
-			print "</body></html>";
+			print "</div></body></html>";
 
 		} else {
 			render_login_form($this->link);
@@ -645,14 +656,14 @@ class Handler_Public extends Handler {
 	}
 
 	function subscribe2() {
-		$feed_url = db_escape_string(trim($_REQUEST["feed_url"]));
-		$cat_id = db_escape_string($_REQUEST["cat_id"]);
-		$from = db_escape_string($_REQUEST["from"]);
+		$feed_url = db_escape_string($this->link, trim($_REQUEST["feed_url"]));
+		$cat_id = db_escape_string($this->link, $_REQUEST["cat_id"]);
+		$from = db_escape_string($this->link, $_REQUEST["from"]);
 
 		/* only read authentication information from POST */
 
-		$auth_login = db_escape_string(trim($_POST["auth_login"]));
-		$auth_pass = db_escape_string(trim($_POST["auth_pass"]));
+		$auth_login = db_escape_string($this->link, trim($_POST["auth_login"]));
+		$auth_pass = db_escape_string($this->link, trim($_POST["auth_pass"]));
 
 		$rc = subscribe_to_feed($this->link, $feed_url, $cat_id, $auth_login, $auth_pass);
 
@@ -733,5 +744,98 @@ class Handler_Public extends Handler {
 		print json_encode(array("error" => array("code" => 7)));
 	}
 
+	function forgotpass() {
+		header('Content-Type: text/html; charset=utf-8');
+		print "<html><head><title>Tiny Tiny RSS</title>";
+
+		print stylesheet_tag("utility.css");
+		print javascript_tag("lib/prototype.js");
+
+		print "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"/>
+			</head><body id='forgotpass'>";
+
+		print '<div class="floatingLogo"><img src="images/logo_small.png"></div>';
+		print "<h1>".__("Password recovery")."</h1>";
+		print "<div class='content'>";
+
+		@$method = $_POST['method'];
+
+		if (!$method) {
+			print_notice(__("You will need to provide valid account name and email. New password will be sent on your email address."));
+
+			print "<form method='POST' action='public.php'>";
+			print "<input type='hidden' name='method' value='do'>";
+			print "<input type='hidden' name='op' value='forgotpass'>";
+
+			print "<fieldset>";
+			print "<label>".__("Login:")."</label>";
+			print "<input type='text' name='login' value='' required>";
+			print "</fieldset>";
+
+			print "<fieldset>";
+			print "<label>".__("Email:")."</label>";
+			print "<input type='email' name='email' value='' required>";
+			print "</fieldset>";
+
+			print "<fieldset>";
+			print "<label>".__("How much is two plus two:")."</label>";
+			print "<input type='text' name='test' value='' required>";
+			print "</fieldset>";
+
+			print "<p/>";
+			print "<button type='submit'>".__("Reset password")."</button>";
+
+			print "</form>";
+		} else if ($method == 'do') {
+
+			$login = db_escape_string($this->link, $_POST["login"]);
+			$email = db_escape_string($this->link, $_POST["email"]);
+			$test = db_escape_string($this->link, $_POST["test"]);
+
+			if (($test != 4 && $test != 'four') || !$email || !$login) {
+				print_error(__('Some of the required form parameters are missing or incorrect.'));
+
+				print "<form method=\"GET\" action=\"public.php\">
+					<input type=\"hidden\" name=\"op\" value=\"forgotpass\">
+					<input type=\"submit\" value=\"".__("Go back")."\">
+					</form>";
+
+			} else {
+
+				$result = db_query($this->link, "SELECT id FROM ttrss_users
+					WHERE login = '$login' AND email = '$email'");
+
+				if (db_num_rows($result) != 0) {
+					$id = db_fetch_result($result, 0, "id");
+
+					Pref_Users::resetUserPassword($this->link, $id, false);
+
+					print "<p>";
+
+					print "<p>"."Completed."."</p>";
+
+					print "<form method=\"GET\" action=\"index.php\">
+						<input type=\"submit\" value=\"".__("Return to Tiny Tiny RSS")."\">
+						</form>";
+
+				} else {
+					print_error(__("Sorry, login and email combination not found."));
+
+					print "<form method=\"GET\" action=\"public.php\">
+						<input type=\"hidden\" name=\"op\" value=\"forgotpass\">
+						<input type=\"submit\" value=\"".__("Go back")."\">
+						</form>";
+
+				}
+			}
+
+		}
+
+		print "</div>";
+		print "</body>";
+		print "</html>";
+
+	}
+
 }
 ?>
