@@ -2,6 +2,7 @@ var notify_silent = false;
 var loading_progress = 0;
 var sanity_check_done = false;
 var init_params = {};
+var _label_base_index = -1024;
 
 Ajax.Base.prototype.initialize = Ajax.Base.prototype.initialize.wrap(
 	function (callOriginal, options) {
@@ -193,23 +194,27 @@ function notify_real(msg, no_hide, n_type) {
 
 	*/
 
-	if (typeof __ != 'undefined') {
-		msg = __(msg);
-	}
+	msg = __(msg);
 
 	if (n_type == 1) {
 		n.className = "notify";
 	} else if (n_type == 2) {
-		n.className = "notifyProgress";
-		msg = "<img src='"+getInitParam("sign_progress")+"'> " + msg;
+		n.className = "notify progress";
+		msg = "<img src='images/indicator_white.gif'> " + msg;
 	} else if (n_type == 3) {
-		n.className = "notifyError";
-		msg = "<img src='"+getInitParam("sign_excl")+"'> " + msg;
+		n.className = "notify error";
+		msg = "<img src='images/sign_excl.svg'> " + msg;
 	} else if (n_type == 4) {
-		n.className = "notifyInfo";
-		msg = "<img src='"+getInitParam("sign_info")+"'> " + msg;
+		n.className = "notify info";
+		msg = "<img src='images/sign_info.svg'> " + msg;
 	}
 
+	if (no_hide) {
+		msg += " <span>(<a href='#' onclick=\"notify('')\">" +
+			__("close") + "</a>)</span>";
+	}
+
+
 //	msg = "<img src='images/live_com_loading.gif'> " + msg;
 
 	nb.innerHTML = msg;
@@ -292,6 +297,10 @@ function gotoPreferences() {
 	document.location.href = "prefs.php";
 }
 
+function gotoLogout() {
+	document.location.href = "backend.php?op=logout";
+}
+
 function gotoMain() {
 	document.location.href = "index.php";
 }
@@ -346,14 +355,14 @@ function toggleSelectListRow2(sender) {
 	return toggleSelectRow(sender, row);
 }
 
-function tSR(sender, row) {
-	return toggleSelectRow(sender, row);
-}
-
 /* this is for dijit Checkbox */
-function toggleSelectRow2(sender, row) {
+function toggleSelectRow2(sender, row, is_cdm) {
 
-	if (!row) row = sender.domNode.parentNode.parentNode;
+	if (!row)
+		if (!is_cdm)
+			row = sender.domNode.parentNode.parentNode;
+		else
+			row = sender.domNode.parentNode.parentNode.parentNode; // oh ffs
 
 	if (sender.checked && !row.hasClassName('Selected'))
 		row.addClassName('Selected');
@@ -406,7 +415,7 @@ function closeInfoBox(cleanup) {
 }
 
 
-function displayDlg(id, param, callback) {
+function displayDlg(title, id, param, callback) {
 
 	notify_progress("Loading, please wait...", true);
 
@@ -416,14 +425,14 @@ function displayDlg(id, param, callback) {
 	new Ajax.Request("backend.php", {
 		parameters: query,
 		onComplete: function (transport) {
-			infobox_callback2(transport);
+			infobox_callback2(transport, title);
 			if (callback) callback(transport);
 		} });
 
 	return false;
 }
 
-function infobox_callback2(transport) {
+function infobox_callback2(transport, title) {
 	try {
 		var dialog = false;
 
@@ -434,13 +443,7 @@ function infobox_callback2(transport) {
 		//console.log("infobox_callback2");
 		notify('');
 
-		var title = transport.responseXML.getElementsByTagName("title")[0];
-		if (title)
-			title = title.firstChild.nodeValue;
-
-		var content = transport.responseXML.getElementsByTagName("content")[0];
-
-		content = content.firstChild.nodeValue;
+		var content = transport.responseText;
 
 		if (!dialog) {
 			dialog = new dijit.Dialog({
@@ -545,28 +548,6 @@ function fatalError(code, msg, ext_info) {
 	}
 }
 
-/* function filterDlgCheckType(sender) {
-
-	try {
-
-		var ftype = sender.value;
-
-		// if selected filter type is 5 (Date) enable the modifier dropbox
-		if (ftype == 5) {
-			Element.show("filterDlg_dateModBox");
-			Element.show("filterDlg_dateChkBox");
-		} else {
-			Element.hide("filterDlg_dateModBox");
-			Element.hide("filterDlg_dateChkBox");
-
-		}
-
-	} catch (e) {
-		exception_error("filterDlgCheckType", e);
-	}
-
-} */
-
 function filterDlgCheckAction(sender) {
 
 	try {
@@ -600,37 +581,9 @@ function filterDlgCheckAction(sender) {
 
 }
 
-function filterDlgCheckDate() {
-	try {
-		var dialog = dijit.byId("filterEditDlg");
-
-		var reg_exp = dialog.attr('value').reg_exp;
-
-		var query = "?op=rpc&method=checkDate&date=" + reg_exp;
-
-		new Ajax.Request("backend.php", {
-			parameters: query,
-			onComplete: function(transport) {
-
-				var reply = JSON.parse(transport.responseText);
-
-				if (reply['result'] == true) {
-					alert(__("Date syntax appears to be correct:") + " " + reply['date']);
-					return;
-				} else {
-					alert(__("Date syntax is incorrect."));
-				}
-
-			} });
-
-
-	} catch (e) {
-		exception_error("filterDlgCheckDate", e);
-	}
-}
 
 function explainError(code) {
-	return displayDlg("explainError", code);
+	return displayDlg(__("Error explained"), "explainError", code);
 }
 
 function loading_set_progress(p) {
@@ -708,15 +661,6 @@ function hotkey_prefix_timeout() {
 	}
 }
 
-function hideAuxDlg() {
-	try {
-		Element.hide('auxDlg');
-	} catch (e) {
-		exception_error("hideAuxDlg", e);
-	}
-}
-
-
 function uploadIconHandler(rc) {
 	try {
 		switch (rc) {
@@ -836,7 +780,7 @@ function addLabel(select, callback) {
 
 function quickAddFeed() {
 	try {
-		var query = "backend.php?op=dlg&method=quickAddFeed";
+		var query = "backend.php?op=feeds&method=quickAddFeed";
 
 		// overlapping widgets
 		if (dijit.byId("batchSubDlg")) dijit.byId("batchSubDlg").destroyRecursive();
@@ -960,6 +904,8 @@ function createNewRuleElement(parentNode, replaceNode) {
 	try {
 		var form = document.forms["filter_new_rule_form"];
 
+		form.reg_exp.value = form.reg_exp.value.replace(/(<([^>]+)>)/ig,"");
+
 		var query = "backend.php?op=pref-filters&method=printrulename&rule="+
 			param_escape(dojo.formToJson(form));
 
@@ -1205,20 +1151,31 @@ function quickAddFilter() {
 			var lh = dojo.connect(dialog, "onLoad", function(){
 				dojo.disconnect(lh);
 
-				var title = $("PTITLE-FULL-" + active_post_id);
+				var query = "op=rpc&method=getlinktitlebyid&id=" + getActiveArticleId();
 
-				if (title || getActiveFeedId() || activeFeedIsCat()) {
-					if (title) title = title.innerHTML;
+				new Ajax.Request("backend.php", {
+				parameters: query,
+				onComplete: function(transport) {
+					var reply = JSON.parse(transport.responseText);
 
-					console.log(title + " " + getActiveFeedId());
+					var title = false;
 
-					var feed_id = activeFeedIsCat() ? 'CAT:' + parseInt(getActiveFeedId()) :
-						getActiveFeedId();
+					if (reply && reply) title = reply.title;
 
-					var rule = { reg_exp: title, feed_id: feed_id, filter_type: 1 };
+					if (title || getActiveFeedId() || activeFeedIsCat()) {
+
+						console.log(title + " " + getActiveFeedId());
+
+						var feed_id = activeFeedIsCat() ? 'CAT:' + parseInt(getActiveFeedId()) :
+							getActiveFeedId();
+
+						var rule = { reg_exp: title, feed_id: feed_id, filter_type: 1 };
+
+						addFilterRule(null, dojo.toJson(rule));
+					}
+
+				} });
 
-					addFilterRule(null, dojo.toJson(rule));
-				}
 			});
 		}
 
@@ -1270,6 +1227,8 @@ function unsubscribeFeed(feed_id, title) {
 					} else {
 						if (feed_id == getActiveFeedId())
 							setTimeout("viewfeed(-5)", 100);
+
+						if (feed_id < 0) updateFeedList();
 					}
 
 				} });
@@ -1314,6 +1273,8 @@ function backend_sanity_check_callback(transport) {
 				for (k in params) {
 					var v = params[k];
 					console.log("IP: " + k + " => " + v);
+
+					if (k == "label_base_index") _label_base_index = parseInt(v);
 				}
 			}
 
@@ -1396,7 +1357,7 @@ function genUrlChangeKey(feed, is_cat) {
 
 			notify_progress("Trying to change address...", true);
 
-			var query = "?op=rpc&method=regenFeedKey&id=" + param_escape(feed) +
+			var query = "?op=pref-feeds&method=regenFeedKey&id=" + param_escape(feed) +
 				"&is_cat=" + param_escape(is_cat);
 
 			new Ajax.Request("backend.php", {
@@ -1624,7 +1585,7 @@ function editFeed(feed, event) {
 
 function feedBrowser() {
 	try {
-		var query = "backend.php?op=dlg&method=feedBrowser";
+		var query = "backend.php?op=feeds&method=feedBrowser";
 
 		if (dijit.byId("feedAddDlg"))
 			dijit.byId("feedAddDlg").hide();
@@ -1733,7 +1694,7 @@ function feedBrowser() {
 					} });
 			},
 			removeFromArchive: function() {
-				var selected = this.getSelectedFeeds();
+				var selected = this.getSelectedFeedIds();
 
 				if (selected.length > 0) {
 
@@ -1742,7 +1703,7 @@ function feedBrowser() {
 					if (confirm(pr)) {
 						Element.show('feed_browser_spinner');
 
-						var query = "?op=rpc&method=remarchived&ids=" +
+						var query = "?op=rpc&method=remarchive&ids=" +
 							param_escape(selected.toString());;
 
 						new Ajax.Request("backend.php", {
@@ -1878,3 +1839,97 @@ function get_timestamp() {
 	var date = new Date();
 	return Math.round(date.getTime() / 1000);
 }
+
+function helpDialog(topic) {
+	try {
+		var query = "backend.php?op=backend&method=help&topic=" + param_escape(topic);
+
+		if (dijit.byId("helpDlg"))
+			dijit.byId("helpDlg").destroyRecursive();
+
+		dialog = new dijit.Dialog({
+			id: "helpDlg",
+			title: __("Help"),
+			style: "width: 600px",
+			href: query,
+		});
+
+		dialog.show();
+
+	} catch (e) {
+		exception_error("helpDialog", e);
+	}
+}
+
+function htmlspecialchars_decode (string, quote_style) {
+  // http://kevin.vanzonneveld.net
+  // +   original by: Mirek Slugen
+  // +   improved by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
+  // +   bugfixed by: Mateusz "loonquawl" Zalega
+  // +      input by: ReverseSyntax
+  // +      input by: Slawomir Kaniecki
+  // +      input by: Scott Cariss
+  // +      input by: Francois
+  // +   bugfixed by: Onno Marsman
+  // +    revised by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
+  // +   bugfixed by: Brett Zamir (http://brett-zamir.me)
+  // +      input by: Ratheous
+  // +      input by: Mailfaker (http://www.weedem.fr/)
+  // +      reimplemented by: Brett Zamir (http://brett-zamir.me)
+  // +    bugfixed by: Brett Zamir (http://brett-zamir.me)
+  // *     example 1: htmlspecialchars_decode("<p>this -&gt; &quot;</p>", 'ENT_NOQUOTES');
+  // *     returns 1: '<p>this -> &quot;</p>'
+  // *     example 2: htmlspecialchars_decode("&amp;quot;");
+  // *     returns 2: '&quot;'
+  var optTemp = 0,
+    i = 0,
+    noquotes = false;
+  if (typeof quote_style === 'undefined') {
+    quote_style = 2;
+  }
+  string = string.toString().replace(/&lt;/g, '<').replace(/&gt;/g, '>');
+  var OPTS = {
+    'ENT_NOQUOTES': 0,
+    'ENT_HTML_QUOTE_SINGLE': 1,
+    'ENT_HTML_QUOTE_DOUBLE': 2,
+    'ENT_COMPAT': 2,
+    'ENT_QUOTES': 3,
+    'ENT_IGNORE': 4
+  };
+  if (quote_style === 0) {
+    noquotes = true;
+  }
+  if (typeof quote_style !== 'number') { // Allow for a single string or an array of string flags
+    quote_style = [].concat(quote_style);
+    for (i = 0; i < quote_style.length; i++) {
+      // Resolve string input to bitwise e.g. 'PATHINFO_EXTENSION' becomes 4
+      if (OPTS[quote_style[i]] === 0) {
+        noquotes = true;
+      } else if (OPTS[quote_style[i]]) {
+        optTemp = optTemp | OPTS[quote_style[i]];
+      }
+    }
+    quote_style = optTemp;
+  }
+  if (quote_style & OPTS.ENT_HTML_QUOTE_SINGLE) {
+    string = string.replace(/&#0*39;/g, "'"); // PHP doesn't currently escape if more than one 0, but it should
+    // string = string.replace(/&apos;|&#x0*27;/g, "'"); // This would also be useful here, but not a part of PHP
+  }
+  if (!noquotes) {
+    string = string.replace(/&quot;/g, '"');
+  }
+  // Put this in last place to avoid escape being double-decoded
+  string = string.replace(/&amp;/g, '&');
+
+  return string;
+}
+
+
+function label_to_feed_id(label) {
+	return _label_base_index - 1 - Math.abs(label);
+}
+
+function feed_to_label_id(feed) {
+	return _label_base_index - 1 + Math.abs(feed);
+}
+
