@@ -2,13 +2,6 @@
 	set_include_path(dirname(__FILE__) ."/include" . PATH_SEPARATOR .
 		get_include_path());
 
-	function __autoload($class) {
-		$file = "classes/".strtolower(basename($class)).".php";
-		if (file_exists($file)) {
-			require $file;
-		}
-	}
-
 	require_once "functions.php";
 	require_once "sessions.php";
 	require_once "sanity_check.php";
@@ -23,7 +16,7 @@
 	$op = $_REQUEST['op'];
 
 	if ($op == "publish"){
-		$key = db_escape_string($_REQUEST["key"]);
+		$key = db_escape_string($link, $_REQUEST["key"]);
 
 		$result = db_query($link, "SELECT owner_uid
 				FROM ttrss_access_keys WHERE
