@@ -1,9 +1,19 @@
 <?php
+	if (file_exists("install") && !file_exists("config.php")) {
+		header("Location: install/");
+	}
+
 	set_include_path(dirname(__FILE__) ."/include" . PATH_SEPARATOR .
 		get_include_path());
 
-	require_once "functions.php";
+	if (!file_exists("config.php")) {
+		print "<b>Fatal Error</b>: You forgot to copy
+		<b>config.php-dist</b> to <b>config.php</b> and edit it.\n";
+		exit;
+	}
+
 	require_once "sessions.php";
+	require_once "functions.php";
 	require_once "sanity_check.php";
 	require_once "version.php";
 	require_once "config.php";
@@ -15,38 +25,63 @@
 
 	login_sequence($link);
 
-	$dt_add = time();
-
 	no_cache_incantation();
 
 	header('Content-Type: text/html; charset=utf-8');
-
 ?>
 <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
 	"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
 <html>
 <head>
 	<title>Tiny Tiny RSS : <?php echo __("Preferences") ?></title>
-	<link rel="stylesheet" type="text/css" href="lib/dijit/themes/claro/claro.css"/>
-	<link rel="stylesheet" type="text/css" href="tt-rss.css?<?php echo $dt_add ?>"/>
 
-	<?php print_theme_includes($link) ?>
+	<?php echo stylesheet_tag("lib/dijit/themes/claro/claro.css"); ?>
+	<?php echo stylesheet_tag("tt-rss.css"); ?>
+	<?php echo stylesheet_tag("prefs.css"); ?>
+
+	<?php if ($_SESSION["uid"]) {
+		$theme = get_pref($link, "USER_CSS_THEME", $_SESSION["uid"], false);
+		if ($theme) {
+			echo stylesheet_tag("themes/$theme");
+		}
+	}
+	?>
+
 	<?php print_user_stylesheet($link) ?>
 
 	<link rel="shortcut icon" type="image/png" href="images/favicon.png"/>
+	<link rel="icon" type="image/png" sizes="72x72" href="images/favicon-72px.png" />
 
-	<script type="text/javascript" src="lib/prototype.js"></script>
-	<script type="text/javascript" src="lib/position.js"></script>
-	<script type="text/javascript" src="lib/scriptaculous/scriptaculous.js?load=effects,dragdrop,controls"></script>
-	<script type="text/javascript" src="lib/dojo/dojo.js"></script>
-	<script type="text/javascript" src="lib/dijit/dijit.js"></script>
-	<script type="text/javascript" src="lib/dojo/tt-rss-layer.js"></script>
+	<?php
+	foreach (array("lib/prototype.js",
+				"lib/scriptaculous/scriptaculous.js?load=effects,dragdrop,controls",
+				"lib/dojo/dojo.js",
+				"lib/dijit/dijit.js",
+				"lib/CheckBoxTree.js",
+				"lib/dojo/tt-rss-layer.js",
+				"errors.php?mode=js") as $jsfile) {
 
-	<script type="text/javascript" charset="utf-8" src="localized_js.php?<?php echo $dt_add ?>"></script>
+		echo javascript_tag($jsfile);
 
-	<script type="text/javascript" charset="utf-8" src="js/functions.js?<?php echo $dt_add ?>"></script>
-	<script type="text/javascript" charset="utf-8" src="js/deprecated.js?<?php echo $dt_add ?>"></script>
-	<script type="text/javascript" charset="utf-8" src="js/prefs.js?<?php echo $dt_add ?>"></script>
+	} ?>
+
+	<script type="text/javascript">
+	<?php
+		require 'lib/jshrink/Minifier.php';
+
+		global $pluginhost;
+
+		foreach ($pluginhost->get_plugins() as $n => $p) {
+			if (method_exists($p, "get_prefs_js")) {
+				echo JShrink\Minifier::minify($p->get_prefs_js());
+			}
+		}
+
+		print get_minified_js(array("functions", "deprecated", "prefs", "PrefFeedTree", "PrefFilterTree", "PrefLabelTree"));
+
+		init_js_translations();
+	?>
+	</script>
 
 	<meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>
 
@@ -73,12 +108,8 @@
 	</div>
 </div>
 
-<div id="hotkey_help_overlay" style="display : none" onclick="Element.hide(this)"></div>
-
-<img id="piggie" src="images/piggie.png" style="display : none" alt="piggie"/>
-
 <div id="header" dojoType="dijit.layout.ContentPane" region="top">
-	<a href='#' onclick="showHelp()"><?php echo __("Keyboard shortcuts") ?></a> |
+	<!-- <a href='#' onclick="showHelp()"><?php echo __("Keyboard shortcuts") ?></a> | -->
 	<a href="#" onclick="gotoMain()"><?php echo __('Exit preferences') ?></a>
 </div>
 
@@ -102,17 +133,19 @@
 		href="backend.php?op=pref-users"
 		title="<?php echo __('Users') ?>"></div>
 <?php } ?>
-<?php if ($_SESSION["access_level"] >= 10 || SINGLE_USER_MODE) { ?>
-	<div id="instanceConfigTab" dojoType="dijit.layout.ContentPane"
-		href="backend.php?op=pref-instances"
-		title="<?php echo __('Linked') ?>"></div>
-<?php } ?>
-
+<?php
+	$pluginhost->run_hooks($pluginhost::HOOK_PREFS_TABS,
+		"hook_prefs_tabs", false);
+?>
 </div>
 
 <div id="footer" dojoType="dijit.layout.ContentPane" region="bottom">
 	<a class="insensitive" target="_blank" href="http://tt-rss.org/">
-	Tiny Tiny RSS</a> &copy; 2005-<?php echo date('Y') ?>
+	Tiny Tiny RSS</a>
+	<?php if (!defined('HIDE_VERSION')) { ?>
+		 v<?php echo VERSION ?>
+	<?php } ?>
+	&copy; 2005-<?php echo date('Y') ?>
 	<a class="insensitive" target="_blank"
 	href="http://fakecake.org/">Andrew Dolgov</a>
 </div> <!-- footer -->
