@@ -17,18 +17,16 @@
 		$_REQUEST = array_map('stripslashes_deep', $_REQUEST);
 	}
 
-	require_once "functions.php";
 	require_once "sessions.php";
+	require_once "functions.php";
 	require_once "sanity_check.php";
 	require_once "config.php";
 	require_once "db.php";
 	require_once "db-prefs.php";
 
-	no_cache_incantation();
-
 	startup_gettext();
 
-	$script_started = getmicrotime();
+	$script_started = microtime(true);
 
 	$link = db_connect(DB_HOST, DB_USER, DB_PASS, DB_NAME);
 
@@ -40,9 +38,16 @@
 
 	$method = $_REQUEST["op"];
 
-	$handler = new Handler_Public($link, $_REQUEST);
+	global $pluginhost;
+	$override = $pluginhost->lookup_handler("public", $method);
 
-	if ($handler->before($method)) {
+	if ($override) {
+		$handler = $override;
+	} else {
+		$handler = new Handler_Public($link, $_REQUEST);
+	}
+
+	if (implements_interface($handler, "IHandler") && $handler->before($method)) {
 		if ($method && method_exists($handler, $method)) {
 			$handler->$method();
 		} else if (method_exists($handler, 'index')) {
