@@ -9,18 +9,40 @@
 
 	require_once "functions.php";
 	require_once "rssfuncs.php";
-	require_once "sanity_check.php";
 	require_once "config.php";
+	require_once "sanity_check.php";
 	require_once "db.php";
 	require_once "db-prefs.php";
-	require_once "update_self.php";
 
 	if (!defined('PHP_EXECUTABLE'))
 		define('PHP_EXECUTABLE', '/usr/bin/php');
 
-	$op = $argv;
+	// Create a database connection.
+	$link = db_connect(DB_HOST, DB_USER, DB_PASS, DB_NAME);
 
-	if (count($argv) == 0 && !defined('STDIN')) {
+	init_connection($link);
+
+	$longopts = array("feeds",
+			"feedbrowser",
+			"daemon",
+			"daemon-loop",
+			"task:",
+			"cleanup-tags",
+			"quiet",
+			"log:",
+			"indexes",
+			"convert-filters",
+			"force-update",
+			"list-plugins",
+			"help");
+
+	foreach ($pluginhost->get_commands() as $command => $data) {
+		array_push($longopts, $command . $data["suffix"]);
+	}
+
+	$options = getopt("", $longopts);
+
+	if (count($options) == 0 && !defined('STDIN')) {
 		?> <html>
 		<head>
 		<title>Tiny Tiny RSS data update script.</title>
@@ -29,7 +51,7 @@
 		</head>
 
 		<body>
-		<div class="floatingLogo"><img src="images/logo_wide.png"></div>
+		<div class="floatingLogo"><img src="images/logo_small.png"></div>
 		<h1><?php echo __("Tiny Tiny RSS data update script.") ?></h1>
 
 		<?php print_error("Please run this script from the command line. Use option \"-help\" to display command help if this error is displayed erroneously."); ?>
@@ -39,32 +61,49 @@
 		exit;
 	}
 
-	if (count($argv) == 1 || in_array("-help", $op) ) {
+	if (count($options) == 0 || isset($options["help"]) ) {
 		print "Tiny Tiny RSS data update script.\n\n";
 		print "Options:\n";
-		print "  -feeds              - update feeds\n";
-		print "  -feedbrowser        - update feedbrowser\n";
-		print "  -daemon             - start single-process update daemon\n";
-		print "  -cleanup-tags       - perform tags table maintenance\n";
-		print "  -get-feeds          - receive popular feeds from linked instances\n";
-		print "  -import USER FILE   - import articles from XML\n";
-		print "  -update-self        - update tt-rss installation to latest version\n";
-		print "  -quiet              - don't show messages\n";
-		print "  -indexes            - recreate missing schema indexes\n";
-		print "  -convert-filters    - convert type1 filters to type2\n";
-		print "  -force-update       - force update of all feeds\n";
-		print "  -help               - show this help\n";
+		print "  --feeds              - update feeds\n";
+		print "  --feedbrowser        - update feedbrowser\n";
+		print "  --daemon             - start single-process update daemon\n";
+		print "  --task N             - create lockfile using this task id\n";
+		print "  --cleanup-tags       - perform tags table maintenance\n";
+		print "  --quiet              - don't output messages to stdout\n";
+		print "  --log FILE           - log messages to FILE\n";
+		print "  --indexes            - recreate missing schema indexes\n";
+		print "  --convert-filters    - convert type1 filters to type2\n";
+		print "  --force-update       - force update of all feeds\n";
+		print "  --list-plugins       - list all available plugins\n";
+		print "  --help               - show this help\n";
+		print "Plugin options:\n";
+
+		foreach ($pluginhost->get_commands() as $command => $data) {
+			$args = $data['arghelp'];
+			printf(" --%-19s - %s\n", "$command $args", $data["description"]);
+		}
+
 		return;
 	}
 
-	define('QUIET', in_array("-quiet", $op));
+	define('QUIET', isset($options['quiet']));
 
-	if (!in_array("-daemon", $op)) {
+	if (isset($options["log"])) {
+		_debug("Logging to " . $options["log"]);
+		define('LOGFILE', $options["log"]);
+	}
+
+	if (!isset($options["daemon"])) {
 		$lock_filename = "update.lock";
 	} else {
 		$lock_filename = "update_daemon.lock";
 	}
 
+	if (isset($options["task"])) {
+		_debug("Using task id " . $options["task"]);
+		$lock_filename = $lock_filename . "-task_" . $options["task"];
+	}
+
 	$lock_handle = make_lockfile($lock_filename);
 	$must_exit = false;
 
@@ -74,12 +113,14 @@
 			"Maybe another update process is already running.\n");
 	}
 
-	// Create a database connection.
-	$link = db_connect(DB_HOST, DB_USER, DB_PASS, DB_NAME);
+	if (isset($options["force-update"])) {
+		_debug("marking all feeds as needing update...");
 
-	init_connection($link);
+		db_query($link, "UPDATE ttrss_feeds SET last_update_started = '1970-01-01',
+				last_updated = '1970-01-01'");
+	}
 
-	if (in_array("-feeds", $op)) {
+	if (isset($options["feeds"])) {
 		// Update all feeds needing a update.
 		update_daemon_common($link);
 
@@ -93,26 +134,28 @@
 		$rc = cleanup_tags($link, 14, 50000);
 		_debug("Cleaned $rc cached tags.");
 
-		get_linked_feeds($link);
+		global $pluginhost;
+		$pluginhost->run_hooks($pluginhost::HOOK_UPDATE_TASK, "hook_update_task", $op);
 	}
 
-	if (in_array("-feedbrowser", $op)) {
+	if (isset($options["feedbrowser"])) {
 		$count = update_feedbrowser_cache($link);
 		print "Finished, $count feeds processed.\n";
 	}
 
-	if (in_array("-daemon", $op)) {
-		$op = array_diff($op, array("-daemon"));
+	if (isset($options["daemon"])) {
 		while (true) {
-			passthru(PHP_EXECUTABLE . " " . implode(' ', $op) . " -daemon-loop");
+			$quiet = (isset($options["quiet"])) ? "--quiet" : "";
+
+			passthru(PHP_EXECUTABLE . " " . $argv[0] ." --daemon-loop $quiet");
 			_debug("Sleeping for " . DAEMON_SLEEP_INTERVAL . " seconds...");
 			sleep(DAEMON_SLEEP_INTERVAL);
 		}
 	}
 
-	if (in_array("-daemon-loop", $op)) {
+	if (isset($options["daemon-loop"])) {
 		if (!make_stampfile('update_daemon.stamp')) {
-			die("error: unable to create stampfile\n");
+			_debug("warning: unable to create stampfile\n");
 		}
 
 		// Call to the feed batch update function
@@ -130,50 +173,18 @@
 
 			_debug("Cleaned $rc cached tags.");
 
-			get_linked_feeds($link);
+			global $pluginhost;
+			$pluginhost->run_hooks($pluginhost::HOOK_UPDATE_TASK, "hook_update_task", $op);
 		}
 
 	}
 
-	if (in_array("-cleanup-tags", $op)) {
+	if (isset($options["cleanup-tags"])) {
 		$rc = cleanup_tags($link, 14, 50000);
 		_debug("$rc tags deleted.\n");
 	}
 
-	if (in_array("-get-feeds", $op)) {
-		get_linked_feeds($link);
-	}
-
-	if (in_array("-import",$op)) {
-		$username = $argv[count($argv) - 2];
-		$filename = $argv[count($argv) - 1];
-
-		if (!$username) {
-			print "error: please specify username.\n";
-			return;
-		}
-
-		if (!is_file($filename)) {
-			print "error: input filename ($filename) doesn't exist.\n";
-			return;
-		}
-
-		_debug("importing $filename for user $username...\n");
-
-		$result = db_query($link, "SELECT id FROM ttrss_users WHERE login = '$username'");
-
-		if (db_num_rows($result) == 0) {
-			print "error: could not find user $username.\n";
-			return;
-		}
-
-		$owner_uid = db_fetch_result($result, 0, "id");
-
-		perform_data_import($link, $filename, $owner_uid);
-
-	}
-
-	if (in_array("-indexes", $op)) {
+	if (isset($options["indexes"])) {
 		_debug("PLEASE BACKUP YOUR DATABASE BEFORE PROCEEDING!");
 		_debug("Type 'yes' to continue.");
 
@@ -228,18 +239,7 @@
 		_debug("all done.");
 	}
 
-	if (in_array("-update-self", $op)) {
-		_debug("Warning: self-updating is experimental. Use at your own risk.");
-		_debug("Please backup your tt-rss directory before continuing. Your database will not be modified.");
-		_debug("Type 'yes' to continue.");
-
-		if (read_stdin() != 'yes')
-			exit;
-
-		update_self($link, in_array("-force", $op));
-	}
-
-	if (in_array("-convert-filters", $op)) {
+	if (isset($options["convert-filters"])) {
 		_debug("WARNING: this will remove all existing type2 filters.");
 		_debug("Type 'yes' to continue.");
 
@@ -290,13 +290,30 @@
 
 	}
 
-	if (in_array("-force-update", $op)) {
-		_debug("marking all feeds as needing update...");
+	if (isset($options["list-plugins"])) {
+		$tmppluginhost = new PluginHost($link);
+		$tmppluginhost->load_all($tmppluginhost::KIND_ALL);
+		$enabled = array_map("trim", explode(",", PLUGINS));
+
+		echo "List of all available plugins:\n";
+
+		foreach ($tmppluginhost->get_plugins() as $name => $plugin) {
+			$about = $plugin->about();
+
+			$status = $about[3] ? "system" : "user";
+
+			if (in_array($name, $enabled)) $name .= "*";
+
+			printf("%-50s %-10s v%.2f (by %s)\n%s\n\n",
+				$name, $status, $about[0], $about[2], $about[1]);
+		}
+
+		echo "Plugins marked by * are currently enabled for all users.\n";
 
-		db_query($link, "UPDATE ttrss_feeds SET last_update_started = '1970-01-01',
-				last_updated = '1970-01-01'");
 	}
 
+	$pluginhost->run_commands($options);
+
 	db_close($link);
 
 	if ($lock_handle != false) {
